﻿import { DataFilterState, DataFilterOptions, DataFilterAction } from "./models";
import { Loop } from "redux-loop";
import createApiCaller from "./createApiCaller";
import { defaultSpecs } from "./defaults";

const createFilterReducer = <TFilterMap, TDomain>(options: DataFilterOptions<TFilterMap[keyof TFilterMap], TDomain>) => {

    type A = DataFilterAction<TFilterMap, TDomain>;
    type FilterState = DataFilterState<TFilterMap[keyof TFilterMap]>;
    type FilterStateOrLoop = FilterState | Loop<FilterState, A>;

    return (state: FilterState, action: A)
        : FilterStateOrLoop => {
        const { filterName } = action;

        

        const callApi = createApiCaller<TFilterMap, TDomain>(filterName);
        if (action.type == "FILTER/INPUT_CHANGED") {
            const oldInput = state.lastApplied ? state.lastApplied.input : {};
            return callApi(state, {
                ...(state.lastApplied || defaultSpecs),
                pageIndex: 0,
                input: {
                    ...oldInput as any,
                    ...action.changes as any
                }
            }, options);
        }
        else if (action.type == "FILTER/PAGE_INDEX_CHANGED") {
            return callApi(state, {
                ...(state.lastApplied || defaultSpecs),
                pageIndex: action.index
            }, options);
        }
        else if (action.type == "FILTER/PAGE_SIZE_CHANGED") {
            return callApi(state, {
                ...(state.lastApplied || defaultSpecs),
                pageIndex: 0,
                pageSize: action.size
            }, options);
        }
        else if (action.type == "FILTER/REFRESH") {
            return callApi(state, {
                ...(state.lastApplied || defaultSpecs)
            }, options);
        }
        else if (action.type == "FILTER/SORT_BY_CHANGED") {
            let sortByDescending = false;
            if (state.lastApplied && state.lastApplied.sortBy == action.path) {
                const oldDescending = state.lastApplied
                    ? !!state.lastApplied.sortByDescending
                    : true;
                sortByDescending = action.descending !== undefined
                    ? action.descending : !oldDescending;
            }

            return callApi(state, {
                ...(state.lastApplied || defaultSpecs),
                pageIndex: 0,
                sortBy: action.path,
                sortByDescending
            }, options);
        }
        else if (action.type == "FILTER/REQUEST_SUCCESS") {
            const { request, payload } = action;
            return state.lastRequest === request
                ? {
                    ...state,
                    isLoading: false,
                    lastApplied: request,
                    total: payload.total,
                    matchKeys: payload.matchKeys,
                } : state;
        }
        else if (action.type == "FILTER/REQUEST_ERROR") {
            const { payload, request } = action;
            return state.lastRequest === request
                ? {
                    ...state,
                    isLoading: false,
                    error: payload,
                } : state;
        }
        else return state;
    }
}

export default createFilterReducer;