﻿import * as React from "react";
import * as ReactDOM from "react-dom";
import { createStore } from 'redux';
import { normalize, schema } from "normalizr";
import { combineReducers, install, StoreCreator } from 'redux-loop';

// test api(s)
import {contactMemoryApi } from "../services/mockBackend";
import {  contacts } from "../services/mockBackend/testData";

// reducer(s)
import { createDataReducer } from "../index";
import { CrmQueries } from "./queryModels";
import { CrmDomain } from "./domainModels";
import { accountEntity, countryEntity, contactEntity } from "./dataNormalizers";
import App from "./App";



const contactApi = contactMemoryApi(1000, contacts);





// setup data reducer and query api(s)
const dataReducer = createDataReducer<CrmQueries, CrmDomain>({
    apiMap: {

        contactsAutoComplete: (specs) => 
            contactApi.autoComplete(specs).then(results => {
                const n = normalize(results.matches, [contactEntity]);
                return {
                    total: results.total,
                    matchKeys: n.result,
                    entities: n.entities
                };
            }),
        getContactById: (specs) =>
            contactApi.findByKey(specs.input.key || "").then(result => {
                const n = normalize(result, contactEntity);
                return {
                    total: 1,
                    matchKeys: n.result,
                    entities: n.entities
                };                
            }
            )

    
    }
});

// create app store
const reducer = combineReducers({
    dataFilters: dataReducer as any
});

export const store = (createStore as StoreCreator)(reducer, undefined as any, install());

// mount application

//ReactDOM.render(<App store={store} />, document.getElementById("root"));
